package net.wanji.datacenter.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.datacenter.kafka.ConsumerHandler;
import net.wanji.datacenter.mapper.*;
import net.wanji.datacenter.pojo.convert.LanePeriodicDataEventListConvert;
import net.wanji.datacenter.pojo.dto.LanePeriodicDataDTO;
import net.wanji.datacenter.pojo.po.*;
import net.wanji.datacenter.service.DataProcessService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author duanruiming
 * @date 2023/03/10 10:30
 * @description: 车道周期实时数据加工处理服务，优化模块实时监控操作数据
 */
@Slf4j
@Service(value = "JN040001LanePeriodicData")
@RequiredArgsConstructor
public class LanePeriodicDataProcessServiceImpl implements DataProcessService {

    private final ConsumerHandler consumerHandler;
    private final CrossBaseLaneInfoMapper crossBaseLaneInfoMapper;
    private final CrossDirDataRealTimeMapper crossDirDataRealTimeMapper;
    private final CrossTurnDataRealTimeMapper crossTurnDataRealTimeMapper;
    private final CrossLaneDataRealTimeMapper crossLaneDataRealTimeMapper;
    private final CrossDataRealTimeMapper crossDataRealTimeMapper;
    private final LanePeriodicDataEventListConvert lanePeriodicDataEventListConvert;

    @Override
    public String receive(String topic) {
        return consumerHandler.getTopicMessage(topic);
    }

    @Override
    public void send(String topic, Object convertData) {
        consumerHandler.removeTopicMessage(topic);
    }

    @Override
    public Object convert(String originalData) throws Exception {
        LanePeriodicDataDTO lanePeriodicDataPO = null;
        try {
            lanePeriodicDataPO = JSONObject.parseObject(originalData, LanePeriodicDataDTO.class);
        } catch (Exception e) {
            log.error("车道周期实时数据转换异常", e);
            throw new Exception();
        }
        return lanePeriodicDataPO;
    }

    @Override
    @Transactional
    public void save(Object convertData) throws Exception {
        LanePeriodicDataDTO lanePeriodicDataDTO = (LanePeriodicDataDTO) convertData;

        int laneNum = lanePeriodicDataDTO.getLaneNum();
        List<CrossDataRealTimePO> crossDataRealTimePOS = new ArrayList<>(laneNum);
        List<CrossDirDataRealTimePO> crossDirDataRealTimePOS = new ArrayList<>(laneNum);
        List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOS = new ArrayList<>(laneNum);
        List<CrossLaneDataRealTimePO> crossLaneDataRealTimePOS = new ArrayList<>(laneNum);

        List<LanePeriodicDataDTO.EventList> eventLists = lanePeriodicDataDTO.getEventList();
        if (CollectionUtils.isEmpty(eventLists)) {
            log.error("当前车道周期实时数据车道列表为空,参数信息：", lanePeriodicDataDTO);
            throw new Exception("当前车道周期实时数据车道列表为空");
        }
        // 通过车道编号获取车道信息
//        List<String> LaneIdList = eventLists.stream().map(LanePeriodicDataDTO.EventList::getLaneId).collect(Collectors.toList());
        // "14PU609IQA014PUG09IRA0010012"
        List<String> LaneIdList = Arrays.asList("14PU609IQA014PUG09IRA0010011");
        List<CrossBaseLaneInfoPO> crossBaseLaneInfoPOS = crossBaseLaneInfoMapper.selectBatchIds(LaneIdList);

        for (LanePeriodicDataDTO.EventList laneRealTimeItem : eventLists) {
            String laneId = laneRealTimeItem.getLaneId();
            for (CrossBaseLaneInfoPO laneInfoPO : crossBaseLaneInfoPOS) {
                String id = laneInfoPO.getId();
                if (StringUtils.endsWithIgnoreCase(laneId, id)) {

                    crossDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossDataRealTimePO(laneRealTimeItem, laneInfoPO));

                    crossDirDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossDirDataRealTimePO(laneRealTimeItem, laneInfoPO));

                    crossTurnDataRealtimePOS.add(lanePeriodicDataEventListConvert.convert2CrossTurnDataRealtimePO(laneRealTimeItem, laneInfoPO));

                    crossLaneDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossLaneDataRealTimePO(laneRealTimeItem, laneInfoPO));

                    String crossId = laneInfoPO.getCrossId();
                    crossDataRealTimeMapper.delete(new LambdaQueryWrapper<CrossDataRealTimePO>().eq(CrossDataRealTimePO::getCrossId, crossId));

                    crossDirDataRealTimeMapper.delete(new LambdaQueryWrapper<CrossDirDataRealTimePO>().eq(CrossDirDataRealTimePO::getCrossId, crossId));

                    crossTurnDataRealTimeMapper.delete(new LambdaQueryWrapper<CrossTurnDataRealtimePO>().eq(CrossTurnDataRealtimePO::getCrossId, crossId));

                    crossLaneDataRealTimeMapper.delete(new LambdaQueryWrapper<CrossLaneDataRealTimePO>().eq(CrossLaneDataRealTimePO::getCrossId, crossId));

                }
            }
        }
        crossDataRealTimeMapper.insertBatch(crossDataRealTimePOS);
        crossDirDataRealTimeMapper.insertBatch(crossDirDataRealTimePOS);
        crossTurnDataRealTimeMapper.insertBatch(crossTurnDataRealtimePOS);
        crossLaneDataRealTimeMapper.insertBatch(crossLaneDataRealTimePOS);
    }
}
